"""
Ship module to create new Ship object and to load, save and delete
ships from remote server.

The uniqueness of ship is represented by the 'MMSI'
(Maritime Mobile Service Identity) field.
There are two types of ships. The ship which already exists
in data store and are compiled form SeaWeb. For these
the 'MMSI' is real ship MMSI.
Second type is artificial ship created by user. This user ships
will start with 9 followed by 8 digits.

In order to be able to work with Mastin cloud services the MMSI is randomly
generated and the user must not change the MMSI by no means.
In most cases the ship will not be validated and such a ship will
not be able to save in remote database neither run any analytics.

You can read more about Maritime MMSI at
https://en.wikipedia.org/wiki/Maritime_Mobile_Service_Identity
"""

import json
import warnings

from mastin.services import ships, auth
from mastin.models.ship import Ship, ShipType
from mastin.utils.utils import random_mmsi


def get_ship(mmsi: int) -> Ship:
    """
    Get ship from database.

    Parameters
    ----------
    mmsi: int
        mmsi of the ship

    Returns
    -------
    ship: Ship

    Raises
    ------
    RuntimeError
        If ship with specified mmsi can not be retrieved from server.
    """
    response = ships.get_ships(access_token=auth.access_token(), mmsi=mmsi)
    if response.status_code == 200:
        return Ship(**json.loads(response.text))
    else:
        raise RuntimeError(f'Can not get Ship with {mmsi}. '
                           f'Server respond: {response.text}')


def create_ship(**kwargs) -> Ship:
    """
    Create ship with specified parameters.

    It is not possible to specify mmsi since this must be unique number.
    Please do not modify the mmsi field it can have bad consequences
    when trying to save it to database.

    Parameters
    ----------
    kwargs:
        Any of the following attributes of :py:class:`Ship`
        in :py:mod:`mastin.models.ship` can be specified.
        Only mmsi will be autogenerated.

    Returns
    -------
    Ship
        Ship object with specified fields. Omitted fields will be set to None.
        The mmsi will be generated randomly as negative integer.
    """

    if 'mmsi' in kwargs:
        warnings.warn('The "mmsi" can not be specified and new '
                      'random "mmsi" will be assigned.')

    kwargs['mmsi'] = random_mmsi()
    return Ship(**kwargs)


def create_standard_ship(ship_type: ShipType) -> Ship:
    """
    Create standard ship per segment (ship type) with all values
    to be calculated as median.

    Parameters
    ----------
    ship_type: ShipType

    Returns
    -------
    Ship

    Raises
    ------
    ValueError
        If ship type is not supported.
    """
    raise NotImplementedError


def create_empty_ship() -> Ship:
    """
    Create ship with only mmsi predefined. All other fields are set to None.

    Returns
    -------
    Ship
    """

    return Ship(mmsi=random_mmsi())


def save(ship: Ship) -> int:
    """
    Save ship to database.

    Parameters
    ----------
    ship: Ship

    Returns
    -------
    MMSI of saved ship.

    Raises
    ------
    RuntimeError
        if ship can not be saved. With server respond to error message.
    """

    response = ships.create_ship(access_token=auth.access_token(),
                                 ship_in=ship.dict())

    if response.status_code == 200:
        return ship.mmsi
    else:
        raise RuntimeError(f'Can not save ship. '
                           f'Server respond: {response.text}')


def delete(mmsi: int) -> int:
    """
    Delete ship from database.

    Parameters
    ----------
    mmsi: int
        The mmsi of the ship.

    Returns
    -------
    MMSI of deleted ship

    Raises
    ------
    RuntimeError
        if ship can not be saved. With server respond to error message.
    """
    response = ships.delete_ship(access_token=auth.access_token(),
                                 mmsi=mmsi)

    if response.status_code == 200:
        return mmsi
    else:
        raise RuntimeError(f'Can not save ship. '
                           f'Server respond: {response.text}')
